use axum::{
    body::Body,
    debug_handler,
    http::header,
    response::{IntoResponse, Response},
    Json,
};
use chrono::NaiveDateTime;
use serde::Deserialize;
use ts_rs::TS;
use utoipa::ToSchema;

use crate::{
    api::{
        description::FILE_TAG,
        routes::{files::sql, AuthBackendType},
    },
    errors::ApiError,
};

#[derive(Debug, Deserialize, TS, ToSchema)]
#[ts(export)]
pub struct GetFileRequest {
    draft_id: String,
    hash: String,
}

#[debug_handler]
#[utoipa::path(
    get,
    path = "/files/draft/file",
    summary = "Download specified file",
    description = "Download specified file.",
    request_body(content = GetFileRequest, description = "Request to download specified file", content_type = "application/json"),
    responses(
        (status = OK, description = "File Data", content_type = "application/octet-stream"),
    ),
     tag = FILE_TAG)]
pub async fn get_specified_draft_file(
    auth_session: AuthBackendType,
    Json(request): Json<GetFileRequest>,
) -> Result<impl IntoResponse, ApiError> {
    let backend = auth_session.backend();
    let user = auth_session
        .is_authenticated()
        .ok_or(ApiError::AccessDenied)?;

    let draft_id =
        NaiveDateTime::parse_from_str(&request.draft_id, "%Y-%m-%d %H:%M:%S").map_err(|_| {
            ApiError::InvalidRequest("Could not parse NaiveDateTime from draft id".to_string())
        })?;

    match sql::get_specified_draft_file(backend.pool(), user, draft_id, request.hash).await? {
        Some(file) => Ok(Response::builder()
            .header(header::CONTENT_TYPE, file.content_type)
            .header(
                header::CONTENT_DISPOSITION,
                format!("attachment; filename=\"{}\"", file.name),
            )
            .body(Body::from(file.data))
            .unwrap_or_default()),
        None => Err(ApiError::FileNotFound),
    }
}
