use axum::{debug_handler, extract::Multipart, http::StatusCode};
use chrono::NaiveDateTime;
use serde::Deserialize;
use tokio_util::bytes::Bytes;
use ts_rs::TS;
use utoipa::ToSchema;

use crate::{
    api::{
        description::FILE_TAG,
        routes::{
            files::{models::File, sql},
            AuthBackendType,
        },
    },
    errors::ApiError,
};

use super::super::FILE_SIZE_LIMIT_MB;

#[derive(Debug, Clone, Deserialize, TS, ToSchema)]
#[ts(export)]
#[allow(unused)]
pub struct FileUploadRequest {
    name: Option<String>,
    #[schema(value_type = String)]
    draft_id: NaiveDateTime,
    #[schema(format = Binary, content_media_type = "application/octet-stream")]
    file: String,
}

#[debug_handler]
#[utoipa::path(
    post,
    path = "/files/draft",
    summary = "Draft File Upload",
    description = "Upload a file as draft file.",
    request_body(content = FileUploadRequest, description = "File Data", content_type = "multipart/form-data"),
    responses(
        (status = OK, body = String, description = "Successfully uploaded and stored file"),
        (status = 413, description = format!("The size of the uploaded file is too large (max {FILE_SIZE_LIMIT_MB} MB)"))
    ),
     tag = FILE_TAG)]
pub async fn upload_draft_file(
    auth_session: AuthBackendType,
    mut multipart: Multipart,
) -> Result<(), ApiError> {
    let backend = auth_session.backend();
    let user = auth_session
        .is_authenticated()
        .ok_or(ApiError::AccessDenied)?;

    let mut name: Option<String> = None;
    let mut draft_id: Option<NaiveDateTime> = None;

    let mut content_type: Option<String> = None;
    let mut file_name: Option<String> = None;
    let mut bytes: Option<Bytes> = None;
    let mut size: Option<i32> = None;

    while let Some(field) = multipart.next_field().await.unwrap() {
        let field_name = field.name();

        match &field_name {
            Some("name") => name = Some(field.text().await?),
            Some("draft_id") => {
                draft_id = Some(
                    NaiveDateTime::parse_from_str(&field.text().await?, "%Y-%m-%d %H:%M:%S")
                        .map_err(|_| {
                            ApiError::InvalidRequest(
                                "Could not parse NaiveDateTime from draft id".to_string(),
                            )
                        })?,
                )
            }
            Some("file") => {
                file_name = field.file_name().map(ToString::to_string);
                content_type = field.content_type().map(ToString::to_string);
                let _bytes = field.bytes().await?;
                size = Some(_bytes.len() as i32);
                bytes = Some(_bytes);
            }
            _ => (),
        };
    }

    // store file in database
    if let (Some(data), Some(draft_id), Some(content_type), Some(filename)) =
        (bytes, draft_id, content_type, file_name)
    {
        sql::insert_new_draft_file(
            backend.pool(),
            user,
            File {
                name: match name {
                    Some(name) => name,
                    None => filename,
                },
                draft_id,
                content_type,
                data,
                size: size.unwrap_or_default(),
            },
        )
        .await
    } else {
        Err(ApiError::MultipartForm(
            StatusCode::BAD_REQUEST,
            "Missing fields in request".to_string(),
        ))
    }
}
