use chrono::NaiveDateTime;
use sqlx::PgPool;

use crate::{api::routes::users::models::User, errors::ApiError};

use super::models::{AttachedFile, File};

pub async fn insert_new_draft_file(pool: &PgPool, user: &User, file: File) -> Result<(), ApiError> {
    let mut transaction = pool.begin().await?;

    let affected_rows = sqlx::query!(
        r#"INSERT INTO draftfiles
        ("UserID", "DraftID", "Hash", "ContentType", "Name", "Data", "Size")
        VALUES ($1, $2, $3, $4, $5, $6, $7) ON CONFLICT DO NOTHING"#,
        user.user_id,
        file.draft_id,
        file.hash(),
        file.content_type,
        file.name,
        &file.data.to_vec(),
        file.size
    )
    .execute(&mut *transaction)
    .await?
    .rows_affected();

    // commit transaction
    transaction.commit().await?;

    Ok(())
}

pub async fn get_draft_attached_files(
    pool: &PgPool,
    user: &User,
    draft_id: NaiveDateTime,
) -> Result<Vec<AttachedFile>, ApiError> {
    Ok(sqlx::query_as!(
        AttachedFile,
        r#"SELECT 
            "Hash" as hash,
            "ContentType" as content_type,
            "Name" as name,
            "Size" as size
        FROM 
            "draftfiles"
        WHERE
            "UserID" = $1 AND "DraftID" = $2
        ORDER BY "Name" ASC"#,
        user.user_id,
        draft_id
    )
    .fetch_all(pool)
    .await?)
}

pub async fn get_specified_draft_file(
    pool: &PgPool,
    user: &User,
    draft_id: NaiveDateTime,
    hash: String,
) -> Result<Option<File>, ApiError> {
    let file = sqlx::query_as!(
        File,
        r#"SELECT 
            "ContentType" as content_type,
            "Name" as name,
            "Data" as data,
            "DraftID" as draft_id,
            "Size" as size
        FROM 
            "draftfiles"
        WHERE
            "UserID" = $1 AND "DraftID" = $2 AND "Hash" = $3
        ORDER BY "Name" ASC"#,
        user.user_id,
        draft_id,
        hash
    )
    .fetch_optional(pool)
    .await?;

    Ok(file)
}
